# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from saharaclient.api import base


class JobBinaries(base.Resource):
    resource_name = 'Job Binary'


class JobBinariesManagerV1(base.ResourceManager):
    resource_class = JobBinaries
    version = 1.1

    def create(self, name, url, description=None, extra=None, is_public=None,
               is_protected=None):
        """Create a Job Binary.

        :param dict extra: authentication info needed for some job binaries,
            containing the keys `user` and `password` for job binary in Swift
            or the keys `accesskey`, `secretkey`, and `endpoint` for job
            binary in S3

        """
        data = {
            "name": name,
            "url": url
        }

        self._copy_if_defined(data, description=description, extra=extra,
                              is_public=is_public, is_protected=is_protected)

        return self._create('/job-binaries', data, 'job_binary')

    def list(self, search_opts=None, limit=None, marker=None,
             sort_by=None, reverse=None):
        """Get a list of Job Binaries."""
        query = base.get_query_string(search_opts, limit=limit, marker=marker,
                                      sort_by=sort_by, reverse=reverse)
        url = "/job-binaries%s" % query
        return self._page(url, 'binaries', limit)

    def get(self, job_binary_id):
        """Get information about a Job Binary."""
        return self._get('/job-binaries/%s' % job_binary_id, 'job_binary')

    def delete(self, job_binary_id):
        """Delete a Job Binary."""
        self._delete('/job-binaries/%s' % job_binary_id)

    def get_file(self, job_binary_id):
        """Download a Job Binary."""
        resp = self.api.get('/job-binaries/%s/data' % job_binary_id)

        if resp.status_code != 200:
            self._raise_api_exception(resp)
        return resp.content

    def update(self, job_binary_id, data):
        """Update Job Binary.

        :param dict data: dict that contains fields that should be updated
                     with new values.

        Fields that can be updated:

        * name
        * description
        * url
        * is_public
        * is_protected
        * extra - dict with the keys `user` and `password` for job binary
          in Swift, or with the keys `accesskey`, `secretkey`, and `endpoint`
          for job binary in S3
        """

        if self.version >= 2:
            UPDATE_FUNC = self._patch
        else:
            UPDATE_FUNC = self._update

        return UPDATE_FUNC(
            '/job-binaries/%s' % job_binary_id, data, 'job_binary')


class JobBinariesManagerV2(JobBinariesManagerV1):
    version = 2


# NOTE(jfreud): keep this around for backwards compatibility
JobBinariesManager = JobBinariesManagerV1



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils
from oslo_log import log as logging
from oslo_serialization import jsonutils

from saharaclient.osc import utils

JOB_FIELDS = ['id', 'job_template_id', 'cluster_id', 'input_id', 'output_id',
              'start_time', 'end_time', 'status', 'is_public', 'is_protected',
              'engine_job_id']

JOB_STATUS_CHOICES = ['done-with-error', 'failed', 'killed', 'pending',
                      'running', 'succeeded', 'to-be-killed']


def _format_job_output(app, data):
    data['status'] = data['info']['status']
    del data['info']
    data['job_template_id'] = data.pop('job_id')


class ExecuteJob(command.ShowOne):
    """Executes job"""

    log = logging.getLogger(__name__ + ".ExecuteJob")

    def get_parser(self, prog_name):
        parser = super(ExecuteJob, self).get_parser(prog_name)

        parser.add_argument(
            '--job-template',
            metavar="<job-template>",
            help="Name or ID of the job template "
                 "[REQUIRED if JSON is not provided]",
        )
        parser.add_argument(
            '--cluster',
            metavar="<cluster>",
            help="Name or ID of the cluster "
                 "[REQUIRED if JSON is not provided]",
        )
        parser.add_argument(
            '--input',
            metavar="<input>",
            help="Name or ID of the input data source",
        )
        parser.add_argument(
            '--output',
            metavar="<output>",
            help="Name or ID of the output data source",
        )
        parser.add_argument(
            '--params',
            metavar="<name:value>",
            nargs='+',
            help="Parameters to add to the job"
        )
        parser.add_argument(
            '--args',
            metavar="<argument>",
            nargs='+',
            help="Arguments to add to the job"
        )
        parser.add_argument(
            '--public',
            action='store_true',
            default=False,
            help='Make the job public',
        )
        parser.add_argument(
            '--protected',
            action='store_true',
            default=False,
            help='Make the job protected',
        )
        configs = parser.add_mutually_exclusive_group()
        configs.add_argument(
            '--config-json',
            metavar='<filename>',
            help='JSON representation of the job configs'
        )
        configs.add_argument(
            '--configs',
            metavar="<name:value>",
            nargs='+',
            help="Configs to add to the job"
        )
        parser.add_argument(
            '--interface',
            metavar='<filename>',
            help='JSON representation of the interface'
        )
        parser.add_argument(
            '--json',
            metavar='<filename>',
            help='JSON representation of the job. Other arguments will not be '
                 'taken into account if this one is provided'
        )
        return parser

    def _take_action(self, client, parsed_args):

        if parsed_args.json:
            blob = osc_utils.read_blob_file_contents(parsed_args.json)
            try:
                template = jsonutils.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'template from file %s: %s' % (parsed_args.json, e))

            if 'job_configs' in template:
                template['configs'] = template.pop('job_configs')

            data = utils.create_job_json(client, self.app, template)
        else:
            if not parsed_args.cluster or not parsed_args.job_template:
                raise exceptions.CommandError(
                    'At least --cluster, --job-template, arguments should be '
                    'specified or json template should be provided with '
                    '--json argument')

            job_configs = {}

            if parsed_args.interface:
                blob = osc_utils.read_blob_file_contents(parsed_args.json)
                try:
                    parsed_args.interface = jsonutils.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'interface from file %s: %s' % (parsed_args.json, e))

            if parsed_args.config_json:
                blob = osc_utils.read_blob_file_contents(parsed_args.configs)
                try:
                    job_configs['configs'] = jsonutils.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'configs from file %s: %s' % (parsed_args.json, e))
            elif parsed_args.configs:
                job_configs['configs'] = dict(
                    map(lambda x: x.split(':', 1), parsed_args.configs))

            if parsed_args.args:
                job_configs['args'] = parsed_args.args

            if parsed_args.params:
                job_configs['params'] = dict(
                    map(lambda x: x.split(':', 1), parsed_args.params))

            jt_id = utils.get_job_template_id(self.app, client, parsed_args)
            cluster_id = utils.get_resource_id(
                client.clusters, parsed_args.cluster)
            if parsed_args.input not in [None, "", "None"]:
                input_id = utils.get_resource_id(
                    client.data_sources, parsed_args.input)
            else:
                input_id = None
            if parsed_args.output not in [None, "", "None"]:
                output_id = utils.get_resource_id(
                    client.data_sources, parsed_args.output)
            else:
                output_id = None

            data = utils.create_job(client, self.app, jt_id, cluster_id,
                                    input_id, output_id, job_configs,
                                    parsed_args)
        sys.stdout.write(
            'Job "{job}" has been started successfully.\n'.format(
                job=data['id']))

        return data

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_job_output(self.app, data)
        data = utils.prepare_data(data, JOB_FIELDS)

        return self.dict2columns(data)


class ListJobs(command.Lister):
    """Lists jobs"""

    log = logging.getLogger(__name__ + ".ListJobs")

    def get_parser(self, prog_name):
        parser = super(ListJobs, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help='List additional fields in output',
        )
        parser.add_argument(
            '--status',
            metavar="<status>",
            choices=JOB_STATUS_CHOICES,
            help="List jobs with specific status"
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = client.job_executions.list()

        for job in data:
            job.status = job.info['status']

        if parsed_args.status:
            data = [job for job in data
                    if job.info['status'] == parsed_args.status.replace(
                        '-', '').upper()]

        if parsed_args.long:
            columns = ('id', 'cluster id', 'job id', 'status', 'start time',
                       'end time')
            column_headers = utils.prepare_column_headers(columns)

        else:
            columns = ('id', 'cluster id', 'job id', 'status')
            column_headers = utils.prepare_column_headers(columns)

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns
            ) for s in data)
        )


class ShowJob(command.ShowOne):
    """Display job details"""

    log = logging.getLogger(__name__ + ".ShowJob")

    def get_parser(self, prog_name):
        parser = super(ShowJob, self).get_parser(prog_name)
        parser.add_argument(
            "job",
            metavar="<job>",
            help="ID of the job to display",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = client.job_executions.get(parsed_args.job).to_dict()

        _format_job_output(self.app, data)
        data = utils.prepare_data(data, JOB_FIELDS)

        return self.dict2columns(data)


class DeleteJob(command.Command):
    """Deletes job"""

    log = logging.getLogger(__name__ + ".DeleteJob")

    def get_parser(self, prog_name):
        parser = super(DeleteJob, self).get_parser(prog_name)
        parser.add_argument(
            "job",
            metavar="<job>",
            nargs="+",
            help="ID(s) of the job(s) to delete",
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            default=False,
            help='Wait for the job(s) delete to complete',
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        for job_id in parsed_args.job:
            client.job_executions.delete(job_id)

            sys.stdout.write(
                'Job "{job}" deletion has been started.\n'.format(job=job_id))

        if parsed_args.wait:
            for job_id in parsed_args.job:
                wait_for_delete = utils.wait_for_delete(
                    client.job_executions, job_id)

                if not wait_for_delete:
                    self.log.error(
                        'Error occurred during job deleting: %s' %
                        job_id)
                else:
                    sys.stdout.write(
                        'Job "{job}" has been removed successfully.\n'.format(
                            job=job_id))


class UpdateJob(command.ShowOne):
    """Updates job"""

    log = logging.getLogger(__name__ + ".UpdateJob")

    def get_parser(self, prog_name):
        parser = super(UpdateJob, self).get_parser(prog_name)

        parser.add_argument(
            'job',
            metavar="<job>",
            help="ID of the job to update",
        )
        public = parser.add_mutually_exclusive_group()
        public.add_argument(
            '--public',
            action='store_true',
            help='Make the job public (Visible from other projects)',
            dest='is_public'
        )
        public.add_argument(
            '--private',
            action='store_false',
            help='Make the job private (Visible only from this project)',
            dest='is_public'
        )
        protected = parser.add_mutually_exclusive_group()
        protected.add_argument(
            '--protected',
            action='store_true',
            help='Make the job protected',
            dest='is_protected'
        )
        protected.add_argument(
            '--unprotected',
            action='store_false',
            help='Make the job unprotected',
            dest='is_protected'
        )

        parser.set_defaults(is_public=None, is_protected=None)

        return parser

    def _take_action(self, client, parsed_args):
        update_dict = utils.create_dict_from_kwargs(
            is_public=parsed_args.is_public,
            is_protected=parsed_args.is_protected)

        data = utils.update_job(client, self.app, parsed_args, update_dict)

        return data

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_job_output(self.app, data)
        data = utils.prepare_data(data, JOB_FIELDS)

        return self.dict2columns(data)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import inspect
import os
import sys

from docutils import nodes
from . import ext


def _get_command(classes):
    """Associates each command class with command depending on setup.cfg
    """
    commands = {}
    setup_file = os.path.join(
        os.path.abspath(os.path.join(os.path.dirname(__file__), '../..')),
        'setup.cfg')
    for line in open(setup_file, 'r'):
        for cl in classes:
            if cl in line:
                commands[cl] = line.split(' = ')[0].strip().replace('_', ' ')

    return commands


class ArgParseDirectiveOSC(ext.ArgParseDirective):
    """Sphinx extension that automatically documents commands and options
    of the module that contains OpenstackClient/cliff command objects

    Usage example:

    .. cli::
       :module: saharaclient.osc.v1.clusters

    """
    def run(self):
        module_name = self.options['module']

        mod = __import__(module_name, globals(), locals())

        classes = inspect.getmembers(sys.modules[module_name], inspect.isclass)
        classes_names = [cl[0] for cl in classes]
        commands = _get_command(classes_names)

        items = []

        for cl in classes:
            parser = cl[1](None, None).get_parser(None)
            parser.prog = commands[cl[0]]
            items.append(nodes.subtitle(text=commands[cl[0]]))
            result = ext.parse_parser(
                parser, skip_default_values='nodefault' in self.options)
            result = ext.parser_navigate(result, '')
            nested_content = ext.nodes.paragraph()
            self.state.nested_parse(
                self.content, self.content_offset, nested_content)
            nested_content = nested_content.children

            for item in nested_content:
                if not isinstance(item, ext.nodes.definition_list):
                    items.append(item)
            if 'description' in result:
                items.append(self._nested_parse_paragraph(result['description']))
            items.append(ext.nodes.literal_block(text=result['usage']))
            items.append(ext.print_command_args_and_opts(
                ext.print_arg_list(result, nested_content),
                ext.print_opt_list(result, nested_content),
                ext.print_subcommand_list(result, nested_content)
            ))
            if 'epilog' in result:
                items.append(self._nested_parse_paragraph(result['epilog']))
        return items


def setup(app):
    app.add_directive('cli', ArgParseDirectiveOSC)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging

from saharaclient.osc import utils
from saharaclient.osc.v1 import clusters as c_v1


def _format_cluster_output(app, data):
    data['image'] = data.pop('default_image_id')
    data['node_groups'] = c_v1._format_node_groups_list(data['node_groups'])
    data['anti_affinity'] = osc_utils.format_list(data['anti_affinity'])


class CreateCluster(c_v1.CreateCluster):
    """Creates cluster"""

    log = logging.getLogger(__name__ + ".CreateCluster")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        if parsed_args.count and parsed_args.count > 1:
            clusters = []
            for cluster in data['clusters']:
                clusters.append(
                    utils.get_resource(client.clusters,
                                       cluster['cluster']['id']))

            if parsed_args.wait:
                for cluster in clusters:
                    if not osc_utils.wait_for_status(
                            client.clusters.get, cluster.id):
                        self.log.error(
                            'Error occurred during cluster creation: %s',
                            data['id'])

            data = {}
            for cluster in clusters:
                data[cluster.name] = cluster.id

        else:
            if parsed_args.wait:
                if not osc_utils.wait_for_status(
                        client.clusters.get, data['id']):
                    self.log.error(
                        'Error occurred during cluster creation: %s',
                        data['id'])
                data = client.clusters.get(data['id']).to_dict()
            _format_cluster_output(self.app, data)
            data = utils.prepare_data(data, c_v1.CLUSTER_FIELDS)

        return self.dict2columns(data)


class ListClusters(c_v1.ListClusters):
    """Lists clusters"""

    log = logging.getLogger(__name__ + ".ListClusters")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        search_opts = {}
        if parsed_args.plugin:
            search_opts['plugin_name'] = parsed_args.plugin
        if parsed_args.plugin_version:
            search_opts['plugin_version'] = parsed_args.plugin_version

        data = client.clusters.list(search_opts=search_opts)

        if parsed_args.name:
            data = utils.get_by_name_substring(data, parsed_args.name)

        if parsed_args.long:
            columns = ('name', 'id', 'plugin_name', 'plugin_version',
                       'status', 'description', 'default_image_id')
            column_headers = utils.prepare_column_headers(
                columns, {'default_image_id': 'image'})
        else:
            columns = ('name', 'id', 'plugin_name', 'plugin_version',
                       'status')
            column_headers = utils.prepare_column_headers(
                columns, {'default_image_id': 'image'})

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns
            ) for s in data)
        )


class ShowCluster(c_v1.ShowCluster):
    """Display cluster details"""

    log = logging.getLogger(__name__ + ".ShowCluster")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data, provision_steps = self._take_action(client, parsed_args)

        _format_cluster_output(self.app, data)

        data = self._show_cluster_info(data, provision_steps, parsed_args)
        return data


class DeleteCluster(c_v1.DeleteCluster):
    """Deletes cluster"""

    log = logging.getLogger(__name__ + ".DeleteCluster")

    def get_parser(self, prog_name):
        parser = super(DeleteCluster, self).get_parser(prog_name)
        parser.add_argument(
            '--force',
            action='store_true',
            default=False,
            help='Force the deletion of the cluster',
        )
        return parser

    def _choose_delete_mode(self, parsed_args):
        if parsed_args.force:
            return "force_delete"
        else:
            return "delete"


class UpdateCluster(c_v1.UpdateCluster):
    """Updates cluster"""

    log = logging.getLogger(__name__ + ".UpdateCluster")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_cluster_output(self.app, data)
        data = utils.prepare_data(data, c_v1.CLUSTER_FIELDS)

        return self.dict2columns(data)


class ScaleCluster(c_v1.ScaleCluster):
    """Scales cluster"""

    log = logging.getLogger(__name__ + ".ScaleCluster")

    def _get_json_arg_helptext(self):
        return '''
               JSON representation of the cluster scale object. Other
               arguments (except for --wait) will not be taken into
               account if this one is provided. Specifiying a JSON
               object is also the only way to indicate specific
               instances to decomission.
               '''

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_cluster_output(self.app, data)
        data = utils.prepare_data(data, c_v1.CLUSTER_FIELDS)

        return self.dict2columns(data)


class VerificationUpdateCluster(c_v1.VerificationUpdateCluster):
    """Updates cluster verifications"""

    log = logging.getLogger(__name__ + ".VerificationUpdateCluster")


class UpdateKeypairCluster(command.ShowOne):
    """Reflects an updated keypair on the cluster"""

    log = logging.getLogger(__name__ + ".UpdateKeypairCluster")

    def get_parser(self, prog_name):
        parser = super(UpdateKeypairCluster, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar="<cluster>",
            help="Name or ID of the cluster",
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        cluster_id = utils.get_resource_id(
            client.clusters, parsed_args.cluster)
        client.clusters.update_keypair(cluster_id)
        sys.stdout.write(
            'Cluster "{cluster}" keypair has been updated.\n'
            .format(cluster=parsed_args.cluster))
        return {}, {}



# Copyright (c) 2016 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import saharaclient
from saharaclient.api import base as api_base
from saharaclient.tests.unit import base


class BaseTest(base.BaseTestCase):
    def test_get_query_string(self):
        res = api_base.get_query_string(None, limit=None, marker=None)
        self.assertEqual("", res)

        res = api_base.get_query_string(None, limit=4, marker=None)
        self.assertEqual("?limit=4", res)

        res = api_base.get_query_string({'opt1': 2}, limit=None, marker=3)
        self.assertEqual("?marker=3&opt1=2", res)

    def test_module_version(self):
        self.assertTrue(hasattr(saharaclient, '__version__'))



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from saharaclient.osc.v1 import job_templates as jt_v1


class CreateJobTemplate(jt_v1.CreateJobTemplate):
    """Creates job template"""

    log = logging.getLogger(__name__ + ".CreateJobTemplate")


class ListJobTemplates(jt_v1.ListJobTemplates):
    """Lists job templates"""

    log = logging.getLogger(__name__ + ".ListJobTemplates")


class ShowJobTemplate(jt_v1.ShowJobTemplate):
    """Display job template details"""

    log = logging.getLogger(__name__ + ".ShowJobTemplate")


class DeleteJobTemplate(jt_v1.DeleteJobTemplate):
    """Deletes job template"""

    log = logging.getLogger(__name__ + ".DeleteJobTemplate")


class UpdateJobTemplate(jt_v1.UpdateJobTemplate):
    """Updates job template"""

    log = logging.getLogger(__name__ + ".UpdateJobTemplate")



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from saharaclient.api import plugins
from saharaclient.tests.unit import base


class PluginTest(base.BaseTestCase):
    body = {
        'description': 'description',
        'name': 'name',
        'version': '1'
    }

    def test_plugins_list(self):
        url = self.URL + '/plugins'
        self.responses.get(url, json={'plugins': [self.body]})

        resp = self.client.plugins.list()

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp[0], plugins.Plugin)
        self.assertFields(self.body, resp[0])

    def test_plugins_get(self):
        url = self.URL + '/plugins/name'
        self.responses.get(url, json={'plugin': self.body})

        resp = self.client.plugins.get('name')
        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, plugins.Plugin)
        self.assertFields(self.body, resp)

    def test_plugins_get_version_details(self):
        url = self.URL + '/plugins/name/1'
        self.responses.get(url, json={'plugin': self.body})

        resp = self.client.plugins.get_version_details('name', '1')
        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, plugins.Plugin)
        self.assertFields(self.body, resp)

    def test_convert_to_cluster_template(self):
        url = self.URL + '/plugins/plugin/1/convert-config/template'
        response = {
            'name': 'name',
            'description': 'description',
            'plugin_name': 'plugin',
            'hadoop_version': '1',
        }
        self.responses.post(url, status_code=202,
                            json={'cluster_template': response})

        resp = self.client.plugins.convert_to_cluster_template(
            'plugin', 1, 'template', 'file')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(response, resp)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os import path
import sys

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils
from oslo_log import log as logging
from oslo_serialization import jsonutils

from saharaclient.osc import utils
from saharaclient.osc.v1.job_templates import JOB_TYPES_CHOICES


class ListJobTypes(command.Lister):
    """Lists job types supported by plugins"""

    log = logging.getLogger(__name__ + ".ListJobTypes")

    def get_parser(self, prog_name):
        parser = super(ListJobTypes, self).get_parser(prog_name)
        parser.add_argument(
            '--type',
            metavar="<type>",
            choices=JOB_TYPES_CHOICES,
            help="Get information about specific job type"
        )
        parser.add_argument(
            '--plugin',
            metavar="<plugin>",
            help="Get only job types supported by this plugin"
        )
        parser.add_argument(
            '--plugin-version',
            metavar="<plugin_version>",
            help="Get only job types supported by specific version of the "
                 "plugin. This parameter will be taken into account only if "
                 "plugin is provided"
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        search_opts = {}
        if parsed_args.type:
            search_opts['type'] = parsed_args.type
        if parsed_args.plugin:
            search_opts['plugin'] = parsed_args.plugin
            if parsed_args.plugin_version:
                search_opts['plugin_version'] = parsed_args.plugin_version
        elif parsed_args.plugin_version:
            raise exceptions.CommandError(
                '--plugin-version argument should be specified with --plugin '
                'argument')

        data = client.job_types.list(search_opts=search_opts)
        for job in data:
            plugins = []
            for plugin in job.plugins:
                versions = ", ".join(sorted(plugin["versions"].keys()))
                if versions:
                    versions = "(" + versions + ")"
                plugins.append(plugin["name"] + versions)
            job.plugins = ', '.join(plugins)

        columns = ('name', 'plugins')
        column_headers = utils.prepare_column_headers(columns)

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns
            ) for s in data)
        )


class GetJobTypeConfigs(command.Command):
    """Get job type configs"""

    log = logging.getLogger(__name__ + ".GetJobTypeConfigs")

    def get_parser(self, prog_name):
        parser = super(GetJobTypeConfigs, self).get_parser(prog_name)
        parser.add_argument(
            "job_type",
            metavar="<job-type>",
            choices=JOB_TYPES_CHOICES,
            help="Type of the job to provide config information about",
        )
        parser.add_argument(
            '--file',
            metavar="<file>",
            help='Destination file (defaults to job type)',
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        if not parsed_args.file:
            parsed_args.file = parsed_args.job_type

        data = client.jobs.get_configs(parsed_args.job_type).to_dict()

        if path.exists(parsed_args.file):
            self.log.error('File "%s" already exists. Choose another one with '
                           '--file argument.' % parsed_args.file)
        else:
            with open(parsed_args.file, 'w') as f:
                jsonutils.dump(data, f, indent=4)
            sys.stdout.write(
                '"%(type)s" job configs were saved in "%(file)s"'
                'file' % {'type': parsed_args.job_type,
                          'file': parsed_args.file})



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from osc_lib import utils as osc_utils
from oslo_log import log as logging

from saharaclient.osc import utils
from saharaclient.osc.v1 import cluster_templates as ct_v1


def _format_ct_output(app, data):
    data['node_groups'] = ct_v1._format_node_groups_list(data['node_groups'])
    data['anti_affinity'] = osc_utils.format_list(data['anti_affinity'])


class CreateClusterTemplate(ct_v1.CreateClusterTemplate):
    """Creates cluster template"""

    log = logging.getLogger(__name__ + ".CreateClusterTemplate")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_ct_output(self.app, data)
        data = utils.prepare_data(data, ct_v1.CT_FIELDS)

        return self.dict2columns(data)


class ListClusterTemplates(ct_v1.ListClusterTemplates):
    """Lists cluster templates"""

    log = logging.getLogger(__name__ + ".ListClusterTemplates")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        search_opts = {}
        if parsed_args.plugin:
            search_opts['plugin_name'] = parsed_args.plugin
        if parsed_args.plugin_version:
            search_opts['plugin_version'] = parsed_args.plugin_version

        data = client.cluster_templates.list(search_opts=search_opts)

        if parsed_args.name:
            data = utils.get_by_name_substring(data, parsed_args.name)

        if parsed_args.long:
            columns = ('name', 'id', 'plugin_name', 'plugin_version',
                       'node_groups', 'description')
            column_headers = utils.prepare_column_headers(columns)

        else:
            columns = ('name', 'id', 'plugin_name', 'plugin_version')
            column_headers = utils.prepare_column_headers(columns)

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns,
                formatters={
                    'node_groups': ct_v1._format_node_groups_list
                }
            ) for s in data)
        )


class ShowClusterTemplate(ct_v1.ShowClusterTemplate):
    """Display cluster template details"""

    log = logging.getLogger(__name__ + ".ShowClusterTemplate")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = utils.get_resource(
            client.cluster_templates, parsed_args.cluster_template).to_dict()

        _format_ct_output(self.app, data)
        data = utils.prepare_data(data, ct_v1.CT_FIELDS)

        return self.dict2columns(data)


class DeleteClusterTemplate(ct_v1.DeleteClusterTemplate):
    """Deletes cluster template"""

    log = logging.getLogger(__name__ + ".DeleteClusterTemplate")


class UpdateClusterTemplate(ct_v1.UpdateClusterTemplate):
    """Updates cluster template"""

    log = logging.getLogger(__name__ + ".UpdateClusterTemplate")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        ct_id = utils.get_resource_id(
            client.cluster_templates, parsed_args.cluster_template)

        data = self._take_action(client, parsed_args, ct_id)

        _format_ct_output(self.app, data)
        data = utils.prepare_data(data, ct_v1.CT_FIELDS)

        return self.dict2columns(data)


class ImportClusterTemplate(ct_v1.ImportClusterTemplate):
    """Imports cluster template"""

    log = logging.getLogger(__name__ + ".ImportClusterTemplate")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_ct_output(self.app, data)
        data = utils.prepare_data(data, ct_v1.CT_FIELDS)

        return self.dict2columns(data)


class ExportClusterTemplate(ct_v1.ExportClusterTemplate):
    """Export cluster template to JSON"""

    log = logging.getLogger(__name__ + ".ExportClusterTemplate")



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging

from saharaclient.osc import utils

NGT_FIELDS = ['id', 'name', 'plugin_name', 'plugin_version', 'node_processes',
              'description', 'auto_security_group', 'security_groups',
              'availability_zone', 'flavor_id', 'floating_ip_pool',
              'volumes_per_node', 'volumes_size',
              'volume_type', 'volume_local_to_instance', 'volume_mount_prefix',
              'volumes_availability_zone', 'use_autoconfig',
              'is_proxy_gateway', 'is_default', 'is_protected', 'is_public']


def _format_ngt_output(data):
    data['node_processes'] = osc_utils.format_list(data['node_processes'])
    data['plugin_version'] = data.pop('hadoop_version')
    if data['volumes_per_node'] == 0:
        del data['volume_local_to_instance']
        del data['volume_mount_prefix']
        del data['volume_type'],
        del data['volumes_availability_zone']
        del data['volumes_size']


class CreateNodeGroupTemplate(command.ShowOne, utils.NodeGroupTemplatesUtils):
    """Creates node group template"""

    log = logging.getLogger(__name__ + ".CreateNodeGroupTemplate")

    def get_parser(self, prog_name):
        parser = super(CreateNodeGroupTemplate, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar="<name>",
            help="Name of the node group template [REQUIRED if JSON is not "
                 "provided]",
        )
        parser.add_argument(
            '--plugin',
            metavar="<plugin>",
            help="Name of the plugin [REQUIRED if JSON is not provided]"
        )
        parser.add_argument(
            '--plugin-version',
            metavar="<plugin_version>",
            help="Version of the plugin [REQUIRED if JSON is not provided]"
        )
        parser.add_argument(
            '--processes',
            metavar="<processes>",
            nargs="+",
            help="List of the processes that will be launched on each "
                 "instance [REQUIRED if JSON is not provided]"
        )
        parser.add_argument(
            '--flavor',
            metavar="<flavor>",
            help="Name or ID of the flavor [REQUIRED if JSON is not provided]"
        )
        parser.add_argument(
            '--security-groups',
            metavar="<security-groups>",
            nargs="+",
            help="List of the security groups for the instances in this node "
                 "group"
        )
        parser.add_argument(
            '--auto-security-group',
            action='store_true',
            default=False,
            help='Indicates if an additional security group should be created '
                 'for the node group',
        )
        parser.add_argument(
            '--availability-zone',
            metavar="<availability-zone>",
            help="Name of the availability zone where instances "
                 "will be created"
        )
        parser.add_argument(
            '--floating-ip-pool',
            metavar="<floating-ip-pool>",
            help="ID of the floating IP pool"
        )
        parser.add_argument(
            '--volumes-per-node',
            type=int,
            metavar="<volumes-per-node>",
            help="Number of volumes attached to every node"
        )
        parser.add_argument(
            '--volumes-size',
            type=int,
            metavar="<volumes-size>",
            help='Size of volumes attached to node (GB). '
                 'This parameter will be taken into account only '
                 'if volumes-per-node is set and non-zero'
        )
        parser.add_argument(
            '--volumes-type',
            metavar="<volumes-type>",
            help='Type of the volumes. '
                 'This parameter will be taken into account only '
                 'if volumes-per-node is set and non-zero'
        )
        parser.add_argument(
            '--volumes-availability-zone',
            metavar="<volumes-availability-zone>",
            help='Name of the availability zone where volumes will be created.'
                 ' This parameter will be taken into account only '
                 'if volumes-per-node is set and non-zero'
        )
        parser.add_argument(
            '--volumes-mount-prefix',
            metavar="<volumes-mount-prefix>",
            help='Prefix for mount point directory. '
                 'This parameter will be taken into account only '
                 'if volumes-per-node is set and non-zero'
        )
        parser.add_argument(
            '--volumes-locality',
            action='store_true',
            default=False,
            help='If enabled, instance and attached volumes will be created on'
                 ' the same physical host. This parameter will be taken into '
                 'account only if volumes-per-node is set and non-zero',
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help='Description of the node group template'
        )
        parser.add_argument(
            '--autoconfig',
            action='store_true',
            default=False,
            help='If enabled, instances of the node group will be '
                 'automatically configured',
        )
        parser.add_argument(
            '--proxy-gateway',
            action='store_true',
            default=False,
            help='If enabled, instances of the node group will be used to '
                 'access other instances in the cluster',
        )
        parser.add_argument(
            '--public',
            action='store_true',
            default=False,
            help='Make the node group template public (Visible from other '
                 'projects)',
        )
        parser.add_argument(
            '--protected',
            action='store_true',
            default=False,
            help='Make the node group template protected',
        )
        parser.add_argument(
            '--json',
            metavar='<filename>',
            help='JSON representation of the node group template. Other '
                 'arguments will not be taken into account if this one is '
                 'provided'
        )
        parser.add_argument(
            '--shares',
            metavar='<filename>',
            help='JSON representation of the manila shares'
        )
        parser.add_argument(
            '--configs',
            metavar='<filename>',
            help='JSON representation of the node group template configs'
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._create_take_action(client, self.app, parsed_args)

        _format_ngt_output(data)
        data = utils.prepare_data(data, NGT_FIELDS)

        return self.dict2columns(data)


class ListNodeGroupTemplates(command.Lister, utils.NodeGroupTemplatesUtils):
    """Lists node group templates"""

    log = logging.getLogger(__name__ + ".ListNodeGroupTemplates")

    def get_parser(self, prog_name):
        parser = super(ListNodeGroupTemplates, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help='List additional fields in output',
        )
        parser.add_argument(
            '--plugin',
            metavar="<plugin>",
            help="List node group templates for specific plugin"
        )

        parser.add_argument(
            '--plugin-version',
            metavar="<plugin_version>",
            help="List node group templates with specific version of the "
                 "plugin"
        )

        parser.add_argument(
            '--name',
            metavar="<name-substring>",
            help="List node group templates with specific substring in the "
                 "name"
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        return self._list_take_action(client, self.app, parsed_args)


class ShowNodeGroupTemplate(command.ShowOne, utils.NodeGroupTemplatesUtils):
    """Display node group template details"""

    log = logging.getLogger(__name__ + ".ShowNodeGroupTemplate")

    def get_parser(self, prog_name):
        parser = super(ShowNodeGroupTemplate, self).get_parser(prog_name)
        parser.add_argument(
            "node_group_template",
            metavar="<node-group-template>",
            help="Name or id of the node group template to display",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = utils.get_resource(
            client.node_group_templates,
            parsed_args.node_group_template).to_dict()

        _format_ngt_output(data)

        data = utils.prepare_data(data, NGT_FIELDS)

        return self.dict2columns(data)


class DeleteNodeGroupTemplate(command.Command, utils.NodeGroupTemplatesUtils):
    """Deletes node group template"""

    log = logging.getLogger(__name__ + ".DeleteNodeGroupTemplate")

    def get_parser(self, prog_name):
        parser = super(DeleteNodeGroupTemplate, self).get_parser(prog_name)
        parser.add_argument(
            "node_group_template",
            metavar="<node-group-template>",
            nargs="+",
            help="Name(s) or id(s) of the node group template(s) to delete",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        for ngt in parsed_args.node_group_template:
            ngt_id = utils.get_resource_id(
                client.node_group_templates, ngt)
            client.node_group_templates.delete(ngt_id)
            sys.stdout.write(
                'Node group template "{ngt}" has been removed '
                'successfully.\n'.format(ngt=ngt))


class UpdateNodeGroupTemplate(command.ShowOne, utils.NodeGroupTemplatesUtils):
    """Updates node group template"""

    log = logging.getLogger(__name__ + ".UpdateNodeGroupTemplate")

    def get_parser(self, prog_name):
        parser = super(UpdateNodeGroupTemplate, self).get_parser(prog_name)

        parser.add_argument(
            'node_group_template',
            metavar="<node-group-template>",
            help="Name or ID of the node group template",
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            help="New name of the node group template",
        )
        parser.add_argument(
            '--plugin',
            metavar="<plugin>",
            help="Name of the plugin"
        )
        parser.add_argument(
            '--plugin-version',
            metavar="<plugin_version>",
            help="Version of the plugin"
        )
        parser.add_argument(
            '--processes',
            metavar="<processes>",
            nargs="+",
            help="List of the processes that will be launched on each "
                 "instance"
        )
        parser.add_argument(
            '--security-groups',
            metavar="<security-groups>",
            nargs="+",
            help="List of the security groups for the instances in this node "
                 "group"
        )
        autosecurity = parser.add_mutually_exclusive_group()
        autosecurity.add_argument(
            '--auto-security-group-enable',
            action='store_true',
            help='Additional security group should be created '
                 'for the node group',
            dest='use_auto_security_group'
        )
        autosecurity.add_argument(
            '--auto-security-group-disable',
            action='store_false',
            help='Additional security group should not be created '
                 'for the node group',
            dest='use_auto_security_group'
        )
        parser.add_argument(
            '--availability-zone',
            metavar="<availability-zone>",
            help="Name of the availability zone where instances "
                 "will be created"
        )
        parser.add_argument(
            '--flavor',
            metavar="<flavor>",
            help="Name or ID of the flavor"
        )
        parser.add_argument(
            '--floating-ip-pool',
            metavar="<floating-ip-pool>",
            help="ID of the floating IP pool"
        )
        parser.add_argument(
            '--volumes-per-node',
            type=int,
            metavar="<volumes-per-node>",
            help="Number of volumes attached to every node"
        )
        parser.add_argument(
            '--volumes-size',
            type=int,
            metavar="<volumes-size>",
            help='Size of volumes attached to node (GB). '
                 'This parameter will be taken into account only '
                 'if volumes-per-node is set and non-zero'
        )
        parser.add_argument(
            '--volumes-type',
            metavar="<volumes-type>",
            help='Type of the volumes. '
                 'This parameter will be taken into account only '
                 'if volumes-per-node is set and non-zero'
        )
        parser.add_argument(
            '--volumes-availability-zone',
            metavar="<volumes-availability-zone>",
            help='Name of the availability zone where volumes will be created.'
                 ' This parameter will be taken into account only '
                 'if volumes-per-node is set and non-zero'
        )
        parser.add_argument(
            '--volumes-mount-prefix',
            metavar="<volumes-mount-prefix>",
            help='Prefix for mount point directory. '
                 'This parameter will be taken into account only '
                 'if volumes-per-node is set and non-zero'
        )
        volumelocality = parser.add_mutually_exclusive_group()
        volumelocality.add_argument(
            '--volumes-locality-enable',
            action='store_true',
            help='Instance and attached volumes will be created on '
                 'the same physical host. This parameter will be taken into '
                 'account only if volumes-per-node is set and non-zero',
            dest='volume_locality'
        )
        volumelocality.add_argument(
            '--volumes-locality-disable',
            action='store_false',
            help='Instance and attached volumes creation on the same physical '
                 'host will not be regulated. This parameter will be taken'
                 'into account only if volumes-per-node is set and non-zero',
            dest='volume_locality'
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help='Description of the node group template'
        )
        autoconfig = parser.add_mutually_exclusive_group()
        autoconfig.add_argument(
            '--autoconfig-enable',
            action='store_true',
            help='Instances of the node group will be '
                 'automatically configured',
            dest='use_autoconfig'
        )
        autoconfig.add_argument(
            '--autoconfig-disable',
            action='store_false',
            help='Instances of the node group will not be '
                 'automatically configured',
            dest='use_autoconfig'
        )
        proxy = parser.add_mutually_exclusive_group()
        proxy.add_argument(
            '--proxy-gateway-enable',
            action='store_true',
            help='Instances of the node group will be used to '
                 'access other instances in the cluster',
            dest='is_proxy_gateway'
        )
        proxy.add_argument(
            '--proxy-gateway-disable',
            action='store_false',
            help='Instances of the node group will not be used to '
                 'access other instances in the cluster',
            dest='is_proxy_gateway'
        )
        public = parser.add_mutually_exclusive_group()
        public.add_argument(
            '--public',
            action='store_true',
            help='Make the node group template public '
                 '(Visible from other projects)',
            dest='is_public'
        )
        public.add_argument(
            '--private',
            action='store_false',
            help='Make the node group template private '
                 '(Visible only from this project)',
            dest='is_public'
        )
        protected = parser.add_mutually_exclusive_group()
        protected.add_argument(
            '--protected',
            action='store_true',
            help='Make the node group template protected',
            dest='is_protected'
        )
        protected.add_argument(
            '--unprotected',
            action='store_false',
            help='Make the node group template unprotected',
            dest='is_protected'
        )
        parser.add_argument(
            '--json',
            metavar='<filename>',
            help='JSON representation of the node group template update '
                 'fields. Other arguments will not be taken into account if '
                 'this one is provided'
        )
        parser.add_argument(
            '--shares',
            metavar='<filename>',
            help='JSON representation of the manila shares'
        )
        parser.add_argument(
            '--configs',
            metavar='<filename>',
            help='JSON representation of the node group template configs'
        )
        parser.set_defaults(is_public=None, is_protected=None,
                            is_proxy_gateway=None, volume_locality=None,
                            use_auto_security_group=None, use_autoconfig=None)
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._update_take_action(client, self.app, parsed_args)

        _format_ngt_output(data)
        data = utils.prepare_data(data, NGT_FIELDS)

        return self.dict2columns(data)


class ImportNodeGroupTemplate(command.ShowOne, utils.NodeGroupTemplatesUtils):
    """Imports node group template"""

    log = logging.getLogger(__name__ + ".ImportNodeGroupTemplate")

    def get_parser(self, prog_name):
        parser = super(ImportNodeGroupTemplate, self).get_parser(prog_name)

        parser.add_argument(
            'json',
            metavar="<json>",
            help="JSON containing node group template",
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            help="Name of the node group template",
        )
        parser.add_argument(
            '--security_groups',
            metavar="<security_groups>",
            help="Security groups of the node group template"
        )
        parser.add_argument(
            '--floating_ip_pool',
            metavar="<floating_ip_pool>",
            help="Floating IP pool of the node group template"
        )
        parser.add_argument(
            '--image_id',
            metavar="<image_id>",
            required=True,
            help="Image ID of the node group template",
        )
        parser.add_argument(
            '--flavor_id',
            metavar="<flavor_id>",
            required=True,
            help="Flavor ID of the node group template",
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._import_take_action(client, parsed_args)

        _format_ngt_output(data)
        data = utils.prepare_data(data, NGT_FIELDS)

        return self.dict2columns(data)


class ExportNodeGroupTemplate(command.Command, utils.NodeGroupTemplatesUtils):
    """Export node group template to JSON"""

    log = logging.getLogger(__name__ + ".ExportNodeGroupTemplate")

    def get_parser(self, prog_name):
        parser = super(ExportNodeGroupTemplate, self).get_parser(prog_name)
        parser.add_argument(
            "node_group_template",
            metavar="<node-group-template>",
            help="Name or id of the node group template to export",
        )
        parser.add_argument(
            "--file",
            metavar="<filename>",
            help="Name of the file node group template should be exported to."
                 "If not provided, print to stdout",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        self._export_take_action(client, parsed_args)



# Copyright (c) 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from osc_lib import utils as osc_utils

from saharaclient.osc import utils
from saharaclient.osc.v1 import node_group_templates as ngt_v1

NGT_FIELDS = ['id', 'name', 'plugin_name', 'plugin_version', 'node_processes',
              'description', 'auto_security_group', 'security_groups',
              'availability_zone', 'flavor_id', 'floating_ip_pool',
              'volumes_per_node', 'volumes_size',
              'volume_type', 'volume_local_to_instance', 'volume_mount_prefix',
              'volumes_availability_zone', 'use_autoconfig',
              'is_proxy_gateway', 'is_default', 'is_protected', 'is_public',
              'boot_from_volume', 'boot_volume_type',
              'boot_volume_availability_zone', 'boot_volume_local_to_instance']


def _format_ngt_output(data):
    data['node_processes'] = osc_utils.format_list(data['node_processes'])
    if data['volumes_per_node'] == 0:
        del data['volume_local_to_instance']
        del data['volume_mount_prefix']
        del data['volume_type'],
        del data['volumes_availability_zone']
        del data['volumes_size']
    if not data['boot_from_volume']:
        del data['boot_volume_type']
        del data['boot_volume_availability_zone']
        del data['boot_volume_local_to_instance']


class CreateNodeGroupTemplate(ngt_v1.CreateNodeGroupTemplate,
                              utils.NodeGroupTemplatesUtils):
    """Creates node group template"""

    def get_parser(self, prog_name):
        parser = super(CreateNodeGroupTemplate, self).get_parser(prog_name)

        parser.add_argument(
            '--boot-from-volume',
            action='store_true',
            default=False,
            help="Make the node group bootable from volume",
        )
        parser.add_argument(
            '--boot-volume-type',
            metavar="<boot-volume-type>",
            help='Type of the boot volume. '
                 'This parameter will be taken into account only '
                 'if booting from volume.'
        )
        parser.add_argument(
            '--boot-volume-availability-zone',
            metavar="<boot-volume-availability-zone>",
            help='Name of the availability zone to create boot volume in.'
                 ' This parameter will be taken into account only '
                 'if booting from volume.'
        )
        parser.add_argument(
            '--boot-volume-local-to-instance',
            action='store_true',
            default=False,
            help='Instance and volume guaranteed on the same host. '
                 'This parameter will be taken into account only '
                 'if booting from volume.'
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._create_take_action(client, self.app, parsed_args)

        _format_ngt_output(data)
        data = utils.prepare_data(data, NGT_FIELDS)

        return self.dict2columns(data)


class ListNodeGroupTemplates(ngt_v1.ListNodeGroupTemplates,
                             utils.NodeGroupTemplatesUtils):
    """Lists node group templates"""

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        return self._list_take_action(client, self.app, parsed_args)


class ShowNodeGroupTemplate(ngt_v1.ShowNodeGroupTemplate,
                            utils.NodeGroupTemplatesUtils):
    """Display node group template details"""

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = utils.get_resource(
            client.node_group_templates,
            parsed_args.node_group_template).to_dict()

        _format_ngt_output(data)

        data = utils.prepare_data(data, NGT_FIELDS)

        return self.dict2columns(data)


class DeleteNodeGroupTemplate(ngt_v1.DeleteNodeGroupTemplate,
                              utils.NodeGroupTemplatesUtils):
    """Deletes node group template"""

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        for ngt in parsed_args.node_group_template:
            ngt_id = utils.get_resource_id(
                client.node_group_templates, ngt)
            client.node_group_templates.delete(ngt_id)
            sys.stdout.write(
                'Node group template "{ngt}" has been removed '
                'successfully.\n'.format(ngt=ngt))


class UpdateNodeGroupTemplate(ngt_v1.UpdateNodeGroupTemplate,
                              utils.NodeGroupTemplatesUtils):
    """Updates node group template"""

    def get_parser(self, prog_name):
        parser = super(UpdateNodeGroupTemplate, self).get_parser(prog_name)

        bootfromvolume = parser.add_mutually_exclusive_group()
        bootfromvolume.add_argument(
            '--boot-from-volume-enable',
            action='store_true',
            help='Makes node group bootable from volume.',
            dest='boot_from_volume'
        )
        bootfromvolume.add_argument(
            '--boot-from-volume-disable',
            action='store_false',
            help='Makes node group not bootable from volume.',
            dest='boot_from_volume'
        )
        parser.add_argument(
            '--boot-volume-type',
            metavar="<boot-volume-type>",
            help='Type of the boot volume. '
                 'This parameter will be taken into account only '
                 'if booting from volume.'
        )
        parser.add_argument(
            '--boot-volume-availability-zone',
            metavar="<boot-volume-availability-zone>",
            help='Name of the availability zone to create boot volume in.'
                 ' This parameter will be taken into account only '
                 'if booting from volume.'
        )
        bfv_locality = parser.add_mutually_exclusive_group()
        bfv_locality.add_argument(
            '--boot-volume-local-to-instance-enable',
            action='store_true',
            help='Makes boot volume explicitly local to instance.',
            dest='boot_volume_local_to_instance'
        )
        bfv_locality.add_argument(
            '--boot-volume-local-to-instance-disable',
            action='store_false',
            help='Removes explicit instruction of boot volume locality.',
            dest='boot_volume_local_to_instance'
        )
        parser.set_defaults(is_public=None, is_protected=None,
                            is_proxy_gateway=None, volume_locality=None,
                            use_auto_security_group=None, use_autoconfig=None,
                            boot_from_volume=None, boot_volume_type=None,
                            boot_volume_availability_zone=None,
                            boot_volume_local_to_instance=None)
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._update_take_action(client, self.app, parsed_args)

        _format_ngt_output(data)
        data = utils.prepare_data(data, NGT_FIELDS)

        return self.dict2columns(data)


class ImportNodeGroupTemplate(ngt_v1.ImportNodeGroupTemplate,
                              utils.NodeGroupTemplatesUtils):
    """Imports node group template"""

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._import_take_action(client, parsed_args)

        _format_ngt_output(data)
        data = utils.prepare_data(data, NGT_FIELDS)

        return self.dict2columns(data)


class ExportNodeGroupTemplate(ngt_v1.ExportNodeGroupTemplate,
                              utils.NodeGroupTemplatesUtils):
    """Export node group template to JSON"""

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        self._export_take_action(client, parsed_args)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from saharaclient.api import jobs
from saharaclient.tests.unit import base

from oslo_serialization import jsonutils as json


class JobTest(base.BaseTestCase):
    body = {
        'name': 'name',
        'type': 'pig',
        'mains': ['job_binary_id'],
        'libs': [],
        'description': 'descr',
        'is_public': True,
        'is_protected': False
    }

    def test_create_job(self):
        url = self.URL + '/jobs'
        self.responses.post(url, status_code=202, json={'job': self.body})

        resp = self.client.jobs.create(**self.body)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(self.body,
                         json.loads(self.responses.last_request.body))
        self.assertIsInstance(resp, jobs.Job)
        self.assertFields(self.body, resp)

    def test_jobs_list(self):
        url = self.URL + '/jobs'
        self.responses.get(url, json={'jobs': [self.body]})

        resp = self.client.jobs.list()

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp[0], jobs.Job)
        self.assertFields(self.body, resp[0])

    def test_jobs_get(self):
        url = self.URL + '/jobs/id'
        self.responses.get(url, json={'job': self.body})

        resp = self.client.jobs.get('id')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, jobs.Job)
        self.assertFields(self.body, resp)

    def test_jobs_get_configs(self):
        url = self.URL + '/jobs/config-hints/Pig'
        response = {
            "job_config": {
                "args": [],
                "configs": []
            },
            "interface": []
        }
        self.responses.get(url, json=response)

        resp = self.client.jobs.get_configs('Pig')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, jobs.Job)
        self.assertFields(response, resp)

    def test_jobs_delete(self):
        url = self.URL + '/jobs/id'
        self.responses.delete(url, status_code=204)

        self.client.jobs.delete('id')

        self.assertEqual(url, self.responses.last_request.url)

    def test_jobs_update(self):
        url = self.URL + '/jobs/id'

        update_body = {
            'name': 'new_name',
            'description': 'description'
        }

        self.responses.patch(url, status_code=202, json=update_body)

        # check that all parameters will be updated
        resp = self.client.jobs.update('id', name='new_name',
                                       description='description')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, jobs.Job)
        self.assertEqual(update_body,
                         json.loads(self.responses.last_request.body))

        # check that parameters will not be updated
        self.client.jobs.update("id")
        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual({},
                         json.loads(self.responses.last_request.body))

        # check that all parameters will be unset
        unset_json = {
            "name": None, "description": None, "is_public": None,
            "is_protected": None
        }

        self.client.jobs.update("id", **unset_json)
        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(unset_json,
                         json.loads(self.responses.last_request.body))



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslo_serialization import jsonutils as json

from saharaclient.api import plugins as api_plugins
from saharaclient.osc.v1 import plugins as osc_plugins
from saharaclient.tests.unit.osc.v1 import fakes


PLUGIN_INFO = {'name': 'fake',
               'title': 'Fake Plugin',
               'versions': ['0.1', '0.2'],
               'description': 'Plugin for tests',
               'required_image_tags': ['fake', '0.1'],
               'node_processes': {
                   'HDFS': ['datanode', 'namenode'],
                   'MapReduce': ['jobtracker', 'tasktracker']
               }, 'plugin_labels': {'enabled': {'status': True}},
               'version_labels': {'0.1': {'enabled': {'status': True}}}}


class TestPlugins(fakes.TestDataProcessing):
    def setUp(self):
        super(TestPlugins, self).setUp()
        self.app.api_version['data_processing'] = '2'
        self.plugins_mock = self.app.client_manager.data_processing.plugins
        self.plugins_mock.reset_mock()


class TestListPlugins(TestPlugins):
    def setUp(self):
        super(TestListPlugins, self).setUp()
        self.plugins_mock.list.return_value = [api_plugins.Plugin(
            None, PLUGIN_INFO)]

        # Command to test
        self.cmd = osc_plugins.ListPlugins(self.app, None)

    def test_plugins_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Versions']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('fake', '0.1, 0.2')]
        self.assertEqual(expected_data, list(data))

    def test_plugins_list_long(self):
        arglist = ['--long']
        verifylist = [('long', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Title', 'Versions', 'Description']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('fake', 'Fake Plugin', '0.1, 0.2',
                          'Plugin for tests')]
        self.assertEqual(expected_data, list(data))


class TestShowPlugin(TestPlugins):
    def setUp(self):
        super(TestShowPlugin, self).setUp()
        self.plugins_mock.get.return_value = api_plugins.Plugin(
            None, PLUGIN_INFO)
        self.plugins_mock.get_version_details.return_value = (
            api_plugins.Plugin(None, PLUGIN_INFO))

        # Command to test
        self.cmd = osc_plugins.ShowPlugin(self.app, None)

    def test_plugin_show(self):
        arglist = ['fake']
        verifylist = [('plugin', 'fake')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.plugins_mock.get.assert_called_once_with('fake')

        # Check that columns are correct
        expected_columns = ('Description', 'Name', 'Title', 'Versions', '',
                            'Plugin version 0.1: enabled', 'Plugin: enabled')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('Plugin for tests', 'fake', 'Fake Plugin',
                         '0.1, 0.2', '', True, True)
        self.assertEqual(expected_data, data)

    def test_plugin_version_show(self):
        arglist = ['fake', '--plugin-version', '0.1']
        verifylist = [('plugin', 'fake'), ('plugin_version', '0.1')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.plugins_mock.get_version_details.assert_called_once_with(
            'fake', '0.1')

        # Check that columns are correct
        expected_columns = ('Description', 'Name', 'Required image tags',
                            'Title', '', 'Plugin version 0.1: enabled',
                            'Plugin: enabled', '', 'Service:', '', 'HDFS',
                            'MapReduce')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('Plugin for tests', 'fake', '0.1, fake',
                         'Fake Plugin', '', True, True, '',
                         'Available processes:', '',
                         'datanode, namenode', 'jobtracker, tasktracker')
        self.assertEqual(expected_data, data)


class TestGetPluginConfigs(TestPlugins):
    def setUp(self):
        super(TestGetPluginConfigs, self).setUp()
        self.plugins_mock.get_version_details.return_value = (
            api_plugins.Plugin(None, PLUGIN_INFO))

        # Command to test
        self.cmd = osc_plugins.GetPluginConfigs(self.app, None)

    @mock.patch('oslo_serialization.jsonutils.dump')
    def test_get_plugin_configs_default_file(self, p_dump):
        m_open = mock.mock_open()
        with mock.patch('builtins.open', m_open, create=True):
            arglist = ['fake', '0.1']
            verifylist = [('plugin', 'fake'), ('plugin_version', '0.1')]

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            self.cmd.take_action(parsed_args)

            # Check that correct arguments were passed
            self.plugins_mock.get_version_details.assert_called_once_with(
                'fake', '0.1')

            args_to_dump = p_dump.call_args[0]
            # Check that the right data will be saved

            self.assertEqual(PLUGIN_INFO, args_to_dump[0])
            # Check that data will be saved to the right file
            self.assertEqual('fake-0.1', m_open.call_args[0][0])

    @mock.patch('oslo_serialization.jsonutils.dump')
    def test_get_plugin_configs_specified_file(self, p_dump):
        m_open = mock.mock_open()
        with mock.patch('builtins.open', m_open):
            arglist = ['fake', '0.1', '--file', 'testfile']
            verifylist = [('plugin', 'fake'), ('plugin_version', '0.1'),
                          ('file', 'testfile')]

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            self.cmd.take_action(parsed_args)

            # Check that correct arguments were passed
            self.plugins_mock.get_version_details.assert_called_once_with(
                'fake', '0.1')

            args_to_dump = p_dump.call_args[0]
            # Check that the right data will be saved

            self.assertEqual(PLUGIN_INFO, args_to_dump[0])
            # Check that data will be saved to the right file
            self.assertEqual('testfile', m_open.call_args[0][0])


class TestUpdatePlugin(TestPlugins):
    def setUp(self):
        super(TestUpdatePlugin, self).setUp()
        self.plugins_mock.update.return_value = api_plugins.Plugin(
            None, PLUGIN_INFO)

        # Command to test
        self.cmd = osc_plugins.UpdatePlugin(self.app, None)

    @mock.patch('osc_lib.utils.read_blob_file_contents')
    def test_plugin_update(self, read):
        arglist = ['fake', 'update.json']
        verifylist = [('plugin', 'fake'), ('json', 'update.json')]
        value = {'plugin_labels': {'enabled': {'status': True}}}
        value = json.dumps(value)
        read.return_value = value
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.plugins_mock.update.assert_called_once_with(
            'fake', {'plugin_labels': {'enabled': {'status': True}}})

        # Check that columns are correct
        expected_columns = ('Description', 'Name', 'Title', 'Versions', '',
                            'Plugin version 0.1: enabled', 'Plugin: enabled')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('Plugin for tests', 'fake', 'Fake Plugin',
                         '0.1, 0.2', '', True, True)
        self.assertEqual(expected_data, data)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from osc_lib import utils as osc_utils
from oslo_log import log as logging

from saharaclient.osc import utils
from saharaclient.osc.v1 import jobs as jobs_v1


def _format_job_output(app, data):
    data['status'] = data['info']['status']
    del data['info']


class ExecuteJob(jobs_v1.ExecuteJob):
    """Executes job"""

    log = logging.getLogger(__name__ + ".ExecuteJob")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_job_output(self.app, data)
        data = utils.prepare_data(data, jobs_v1.JOB_FIELDS)

        return self.dict2columns(data)


class ListJobs(jobs_v1.ListJobs):
    """Lists jobs"""

    log = logging.getLogger(__name__ + ".ListJobs")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = client.jobs.list()

        for job in data:
            job.status = job.info['status']

        if parsed_args.status:
            data = [job for job in data
                    if job.info['status'] == parsed_args.status.replace(
                        '-', '').upper()]

        if parsed_args.long:
            columns = ('id', 'cluster id', 'job template id', 'status',
                       'start time', 'end time')
            column_headers = utils.prepare_column_headers(columns)

        else:
            columns = ('id', 'cluster id', 'job template id', 'status')
            column_headers = utils.prepare_column_headers(columns)

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns
            ) for s in data)
        )


class ShowJob(jobs_v1.ShowJob):
    """Display job details"""

    log = logging.getLogger(__name__ + ".ShowJob")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = client.jobs.get(parsed_args.job).to_dict()

        _format_job_output(self.app, data)
        data = utils.prepare_data(data, jobs_v1.JOB_FIELDS)

        return self.dict2columns(data)


class DeleteJob(jobs_v1.DeleteJob):
    """Deletes job"""

    log = logging.getLogger(__name__ + ".DeleteJob")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        for job_id in parsed_args.job:
            client.jobs.delete(job_id)
            sys.stdout.write(
                'Job "{job}" deletion has been started.\n'.format(job=job_id))

        if parsed_args.wait:
            for job_id in parsed_args.job:
                wait_for_delete = utils.wait_for_delete(client.jobs, job_id)

                if not wait_for_delete:
                    self.log.error(
                        'Error occurred during job deleting: %s' %
                        job_id)
                else:
                    sys.stdout.write(
                        'Job "{job}" has been removed successfully.\n'.format(
                            job=job_id))


class UpdateJob(jobs_v1.UpdateJob):
    """Updates job"""

    log = logging.getLogger(__name__ + ".UpdateJob")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_job_output(self.app, data)
        data = utils.prepare_data(data, jobs_v1.JOB_FIELDS)

        return self.dict2columns(data)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from saharaclient.osc.v1 import images as images_v1

IMAGE_FIELDS = ['name', 'id', 'username', 'tags', 'status', 'description']


class ListImages(images_v1.ListImages):
    """Lists registered images"""

    log = logging.getLogger(__name__ + ".ListImages")


class ShowImage(images_v1.ShowImage):
    """Display image details"""

    log = logging.getLogger(__name__ + ".ShowImage")


class RegisterImage(images_v1.RegisterImage):
    """Register an image"""

    log = logging.getLogger(__name__ + ".RegisterImage")


class UnregisterImage(images_v1.UnregisterImage):
    """Unregister image(s)"""

    log = logging.getLogger(__name__ + ".RegisterImage")


class SetImageTags(images_v1.SetImageTags):
    """Set image tags (Replace current image tags with provided ones)"""

    log = logging.getLogger(__name__ + ".AddImageTags")


class AddImageTags(images_v1.AddImageTags):
    """Add image tags"""

    log = logging.getLogger(__name__ + ".AddImageTags")


class RemoveImageTags(images_v1.RemoveImageTags):
    """Remove image tags"""

    log = logging.getLogger(__name__ + ".RemoveImageTags")



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from saharaclient.osc.v1 import plugins as p_v1


class ListPlugins(p_v1.ListPlugins):
    """Lists plugins"""

    log = logging.getLogger(__name__ + ".ListPlugins")


class ShowPlugin(p_v1.ShowPlugin):
    """Display plugin details"""

    log = logging.getLogger(__name__ + ".ShowPlugin")


class GetPluginConfigs(p_v1.GetPluginConfigs):
    """Get plugin configs"""

    log = logging.getLogger(__name__ + ".GetPluginConfigs")


class UpdatePlugin(p_v1.UpdatePlugin):
    log = logging.getLogger(__name__ + ".UpdatePlugin")
